
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.030.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Read, Write file meta-data",
  "dist_author" => [
    "Kathryn Andersen <perlkat\@katspace.org>"
  ],
  "dist_name" => "File-Sticker",
  "dist_version" => "3.0101",
  "license" => "perl",
  "module_name" => "File::Sticker",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "DBD::SQLite::Constants" => 0,
    "DBI" => 0,
    "File::Basename" => 0,
    "File::ExtAttr" => 0,
    "File::LibMagic" => 0,
    "File::Spec" => 0,
    "Getopt::ArgvFile" => 0,
    "Getopt::Long" => 0,
    "Hash::Merge" => 0,
    "Image::ExifTool" => 0,
    "List::MoreUtils" => 0,
    "MP3::Tag" => 0,
    "Module::Pluggable" => 0,
    "POSIX" => 0,
    "Path::Tiny" => 0,
    "Pod::Usage" => 0,
    "Search::Query" => 0,
    "String::CamelCase" => 0,
    "YAML::Any" => 0,
    "common::sense" => 0,
    "parent" => 0,
    "perl" => "v5.10.0"
  },
  "script_files" => [
    "scripts/sticker"
  ],
  "test_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "File::Spec" => 0,
    "IO::Handle" => 0,
    "IPC::Open3" => 0,
    "Test::More" => 0,
    "strict" => 0,
    "warnings" => 0
  }
);


my %fallback_build_requires = (
  "ExtUtils::MakeMaker" => 0,
  "File::Spec" => 0,
  "IO::Handle" => 0,
  "IPC::Open3" => 0,
  "Module::Build" => "0.28",
  "Test::More" => 0,
  "strict" => 0,
  "warnings" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
